using System;
using System.Globalization;
using System.Xml;

namespace jachymko.Nyx {
	public struct Friend : IComparable {
		private string userName;
		private string realName;
		private DateTime lastRefresh;
		private int id;

		public static readonly Friend Empty = new Friend();

		public Friend(XmlNode node) {
			this.userName = string.Empty;
			this.realName = string.Empty;
			this.lastRefresh = DateTime.MinValue;
			this.id = 0;

			if(node == null) {
				throw new ArgumentNullException("node");
			}

			for(int i = 0; i < node.ChildNodes.Count; i++) {
				switch(node.ChildNodes[i].Name) {
					case "username":
						this.userName = node.ChildNodes[i].InnerText;
						break;
					case "name":
						this.realName = node.ChildNodes[i].InnerText;
						break;
					case "refresh":
						double time = 0;

						if(double.TryParse(node.ChildNodes[i].InnerText, NumberStyles.Integer, CultureInfo.InvariantCulture, out time)) {
							this.lastRefresh = new DateTime(1970, 1, 1).AddSeconds(time).ToLocalTime(); // from unix time
						}
						break;
					case "id":
						this.id = int.Parse(node.ChildNodes[i].InnerText);
						break;
					default:
						break;
				}
			}
		}

		public string UserName {
			get {
				return this.userName.ToUpper();
			}
		}
		public string RealName {
			get {
				return this.realName;
			}
		}
		public DateTime LastRefresh {
			get {
				return this.lastRefresh;
			}
		}
		public int Id {
			get {
				return id;
			}
		}
		public int CompareTo(object obj) {
			return UserName.CompareTo(((Friend)obj).UserName);
		}
		public override string ToString() {
			return UserName;
		}
		public override bool Equals(object obj) {
			return CompareTo(obj) == 0;
		}
		public override int GetHashCode() {
			return id;
		}
	}
}
