using System;
using System.Globalization;
using System.Xml;

namespace jachymko.Nyx
{
	[Serializable]
	public class Mail : IComparable {
		private string userName;
		private string realName;
		private string text;
		private DateTime time;

		public Mail(string from, string text, DateTime time) {
			this.userName = from;
			this.text = text;
			this.time = time;
		}

		public Mail(XmlNode node) {
			this.userName = string.Empty;
			this.realName = string.Empty;
			this.text = string.Empty;
			this.time = DateTime.MinValue;

			if(node == null) {
				throw new ArgumentNullException("node");
			}

			for(int i = 0; i < node.ChildNodes.Count; i++) {
				switch(node.ChildNodes[i].Name) {
					case "username":
						this.userName = node.ChildNodes[i].InnerText;
						break;
					case "name":
						this.realName = node.ChildNodes[i].InnerText;
						break;
					case "time":
						double seconds = 0;

						if(double.TryParse(node.ChildNodes[i].InnerText, NumberStyles.Integer, CultureInfo.InvariantCulture, out seconds)) {
							this.time = new DateTime(1970, 1, 1).AddSeconds(seconds).ToLocalTime();
						}
						break;
					case "text":
						this.text = node.ChildNodes[i].InnerText;
						break;
					default:
						break;
				}
			}
		}

		public string Text {
			get {
				return this.text;
			}
		}
		public string UserName {
			get {
				return this.userName.ToUpper();
			}
		}
		public string RealName {
			get {
				return this.realName;
			}
		}
		public DateTime Time {
			get {
				return this.time;
			}
		}
		public int CompareTo(object obj) {
			Mail m = (Mail)obj;

			return time.CompareTo(m.time);
		}
		public override string ToString() {
			return string.Format("{0} --- {1} {2}", userName, time.ToShortTimeString(),
				time.ToShortDateString());
		}
		public override bool Equals(object obj) {
			Mail m = (Mail)obj;

			return (m.time == time) && (m.userName.ToLower() == userName.ToLower());
		}
		public override int GetHashCode() {
			return time.GetHashCode() ^ userName.GetHashCode();
		}
	}


}
