using System;
using System.Text;
using jachymko.Nyx;

namespace jachymko.Nyx.ConsoleClient {
	class Startup {

		// arguments
		static bool cz, nf, nm, nb;
		static int r = 10000;

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static int Main(string[] args) {
			
			Argument.Parser.UseGeneratedHelp = true;
			Argument.Parser ap = new Argument.Parser();
			
			ap.UsageString = ap.UsageString.Replace("[OPTION]", "[options] [username] [password]");
			ap.ProgramDescription = "nyxc connects to the external interface of nyx.cz and notifies you on new \n"    + 
									"mail messages, write-ups in your bookmarks and when your friends connect and \n" +
									"disconnect. You don't have to specify username and password, if you set \n"      +
									"env. variables NYX_USER and NYX_PWD approprietly.\n";

			ap.MoreArgsAllowed = true;
			ap.MinRequiredArgCount = 0;

			ap.Flags.Add("leavecz", "leavecz", "cz", "Do not remove czech characters");
			ap.Flags.Add("nofriends", "nofriends", "nf", "Do not notify when a friend connects/disconnects");
			ap.Flags.Add("nomail", "nomail", "nm", "Do not notify when a message arrives");
			ap.Flags.Add("nobook", "nobook", "nb", "Do not notify on new write-ups in booked topics");
			ap.Flags.Add("refresh", "refresh", "r", true, "10", "Delay in seconds between each refresh");

			string[] extra;
			string user, pwd;

			WriteLogo();

			if(!ap.Parse(args, out extra)) {
				return -1;
			}

			// if arguments passed, use them; otherwise try to use env. variables
			if(extra.Length == 2) {
				user = extra[0];
				pwd = extra[1];
			} else {
				user = Environment.GetEnvironmentVariable("NYX_USER");
				pwd = Environment.GetEnvironmentVariable("NYX_PWD");

				if(user == null || pwd == null) {
					ap.DisplayUsage();
					return -1;
				}
			}

			cz = ap.Flags["leavecz"].IsSet;
			nf = ap.Flags["nofriends"].IsSet;
			nb = ap.Flags["nobook"].IsSet;
			nm = ap.Flags["nomail"].IsSet;

			if(ap.Flags["refresh"].IsSet) {
				try {
					r = 1000 * int.Parse( ap.Flags["refresh"].Value );
					if(r < 10000) throw new Exception();
				} catch {
					Console.WriteLine("Refresh interval must be a number greater than 10 (seconds)");
					return -3;
				}
			}

			Nyx.Witch witch = new Nyx.Witch(user, pwd, r);
			
			try {
				witch.Open();
			} catch(WrongPasswordException) {
				Console.WriteLine("Wrong password or username");
				return -2;
			}

			witch.FriendSignedIn  += new FriendEventHandler(witch_FriendSignedIn);
			witch.FriendSignedOut += new FriendEventHandler(witch_FriendSignedOut);
			witch.NewMail         += new NewMailEventHandler(witch_NewMail);
			witch.NewWriteUp	  += new NewWriteUpEventHandler(witch_NewWriteUp);

			
			Console.WriteLine("Currently active friends:");
			
			foreach(Friend f in witch.ActiveFriends) {
				Console.WriteLine("   " + DateTime.Now.ToString("G") + " " + f.UserName);
			}


			while(true) {
				System.Threading.Thread.Sleep(10000);
			}
		}

		private static void witch_FriendSignedIn(object sender, jachymko.Nyx.FriendEventArgs args) {
			if(nf) return;

			Console.WriteLine(" + " + DateTime.Now.ToString("G") + " " + args.Friend.ToString() + " is online");
		}

		private static void witch_FriendSignedOut(object sender, jachymko.Nyx.FriendEventArgs args) {
			if(nf) return;

			Console.WriteLine(" - " + DateTime.Now.ToString("G") + " " + args.Friend.ToString() + " is offline");
		}

		private static void witch_NewMail(object sender, jachymko.Nyx.NewMailEventArgs args) {
			if(nm) return;

			Console.WriteLine("MAAAAAAAAAAAAAAAAAAAAAIIIIIIIIIIIIIIIIIIIIIIIILLLLLLLLLLLLLLL");
			Console.WriteLine("FROM: {0} --- --- {1} {2}", args.Mail.UserName, args.Mail.Time.ToShortTimeString(),
				args.Mail.Time.ToShortDateString());
			
			
			Console.WriteLine(StripTagsAndCzChars(args.Mail.Text));
			Console.WriteLine("=============================================================");
		}

		private static void witch_NewWriteUp(object sender, NewWriteUpEventArgs args) {
			if(nb) return;

			Console.WriteLine(" * " + DateTime.Now.ToString("G") + " New write-up in " + StripTagsAndCzChars(args.Topic.Name));
		}

		// add more if necessary
		private static readonly char[] badChars =  new char[] { '','','','','','','','','','','','','','','',
																'','','','','','','','','','','','','','','',
																  
																'', ''
															  };

		private static readonly char[] goodChars = new char[] { 'z','s','c','r','d','t','n','a','e','e','i','o','u','u','y',
																'Z','S','C','R','D','T','N','A','E','E','I','O','U','U','Y' 
															  };
		private static string StripTagsAndCzChars(string s) {
			
			if(cz) return s;

			StringBuilder sb = new StringBuilder();
			bool inTag = false;

			for(int i = 0; i < s.Length; i++) {
				if(s[i] == '<') {
					inTag = true;
				} else  if(s[i] == '>') {
					inTag = false;
				}
				
				if(inTag) continue;

				int badIdx = Array.IndexOf(badChars, s[i]);
				if(badIdx < 0) {
					sb.Append(s[i]);
				} else if(badIdx < goodChars.Length) {
					sb.Append(goodChars[badIdx]);
				}
			}

			return sb.ToString();
		}

		private static void WriteLogo() {
			Console.WriteLine("Nyx.cz console client. Copyright (c) 2003-2004 by jachymko\n");
		}

	}
}
