using System;
using System.Collections;
using System.Xml;

namespace jachymko.Nyx.Wokno {

	public class AllFriends {

		private class _Friend : IComparable {
			public string userName;
			public bool notifyOnline;
			public bool notifyOffline;

			internal _Friend() {
				userName = string.Empty;
				notifyOnline = notifyOffline = true;
			}
			internal _Friend(XmlNode n) : this() {
				if(n.Attributes["userName"] != null) userName = n.Attributes["userName"].InnerText.ToUpper();
				if(n.Attributes["notifyOnline"] != null) notifyOnline = bool.Parse(n.Attributes["notifyOnline"].InnerText);
				if(n.Attributes["notifyOffline"] != null) notifyOffline = bool.Parse(n.Attributes["notifyOffline"].InnerText);
			}

			public override bool Equals(object obj) {
				return CompareTo(obj) == 0;
			}

			public override string ToString() {
				return userName.ToUpper();
			}

			public override int GetHashCode() {
				return userName.GetHashCode();
			}

			public int CompareTo(object obj) {
				return  userName.ToUpper().CompareTo(((_Friend)obj).userName.ToUpper());
			}

			internal void Save(XmlTextWriter xtw) {
				xtw.WriteStartElement("friend");

				xtw.WriteAttributeString("userName", userName.ToUpper());
				xtw.WriteAttributeString("notifyOnline", notifyOnline.ToString());
				xtw.WriteAttributeString("notifyOffline", notifyOffline.ToString());

				xtw.WriteEndElement();
			}

		}

		private ArrayList allFriends = new ArrayList();

		internal AllFriends(XmlNode node) {
			XmlNodeList nodes = node.SelectNodes("friend");
			if(nodes != null && nodes.Count > 0) {
				
				_Friend[] fs = new _Friend[nodes.Count];
				for(int i = 0; i < nodes.Count; i++) {
					fs[i] = new _Friend(nodes[i]);
				}
				Array.Sort(fs);
				allFriends.AddRange(fs);
			}
		}

		internal AllFriends() {
		}

		public string this[int index] {
			get {
				return ((_Friend)allFriends[index]).userName.ToUpper();
			}
		}

		public int IndexOf(string userName) {
			for(int i = 0; i < allFriends.Count; i++) {
				if(this[i] == userName.ToUpper()) {
					return i;
				}
			}

			return -1;
		}

		public bool GetNotifyOnline(int idx) {
			if(idx > -1) {
				return ((_Friend)allFriends[idx]).notifyOnline;
			} else {
				return Startup.Config.NotifyUnknownOnline;
			}
		}

		public bool GetNotifyOnline(string userName) {
			return GetNotifyOnline(IndexOf(userName));
		}

		public bool GetNotifyOffline(int idx) {
			if(idx > -1) {
				return ((_Friend)allFriends[idx]).notifyOffline;
			} else {
				return Startup.Config.NotifyUnknownOffline;
			}
		}

		public bool GetNotifyOffline(string userName) {
			return GetNotifyOffline(IndexOf(userName));
		}

		public void SetNotifyOnline(string userName, bool value) {
			int idx = IndexOf(userName);
			
			if(idx > -1) {
				((_Friend)allFriends[idx]).notifyOnline = value;
			} 
		}

		public void SetNotifyOffline(string userName, bool value) {
			int idx = IndexOf(userName);
			
			if(idx > -1) {
				((_Friend)allFriends[idx]).notifyOffline = value;
			} 
		}

		public void Add(string friendName) {
			if(IndexOf(friendName) > -1) return;

			_Friend f = new _Friend();
			f.userName = friendName.ToUpper();

			allFriends.Add(f);
		}

		public void AddRange(object[] friends) {
			for(int i = 0; i < friends.Length; i++) {
				Add(friends[i].ToString());
			}
		}

		internal void Save(XmlTextWriter xtw) {
			xtw.WriteStartElement("friends");

			for(int i = 0; i < allFriends.Count; i++) {
				((_Friend)allFriends[i]).Save(xtw);
			}

			xtw.WriteEndElement();
		}

		public int Count {
			get {
				return allFriends.Count;
			}
		}
	}
}
