using System;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace jachymko.Nyx.Wokno {

	public class AutoAway {
	
		#region P/Invoke
		[DllImport("user32.dll")]
		private static extern IntPtr OpenInputDesktop(int dwFlags, bool fInherit, int dwDesiredAccess);
		
		[DllImport("user32.dll")]
		private static extern bool CloseDesktop(IntPtr hDesktop);
		
		[DllImport("user32.dll")]
		private static extern bool GetUserObjectInformation(
			IntPtr hObj, 
			int nIndex,
			[MarshalAs(UnmanagedType.LPStr)] StringBuilder pvInfo,
			int nLength, 
			IntPtr lpnLengthNeeded);

		[DllImport("user32.dll")]
		private static extern bool SystemParametersInfo(
			uint uiAction,
			uint uiParam,
			ref bool pvParam,
			bool fWinIni);

		[DllImport("user32.dll")]
		private static extern bool GetLastInputInfo(out LASTINPUTINFO plii);

		[StructLayout(LayoutKind.Sequential)]
		private struct LASTINPUTINFO {
			public uint cbSize;
			public int dwTime;
		}

		private const int MAXIMUM_ALLOWED = 0x02000000;
		private const int UOI_NAME = 2;
		private const int SPI_GETSCREENSAVERRUNNING = 0x0072;
		#endregion

		

		private AutoAway() { }

		public static bool IsWorkstationLocked {
			get {
				if(Environment.OSVersion.Platform != PlatformID.Win32NT) {
					return false;
				}

				IntPtr hDesktop = OpenInputDesktop(0, false, MAXIMUM_ALLOWED);
	
				if(hDesktop == IntPtr.Zero) {
					System.Diagnostics.Debug.WriteLine("hDesktop == IntPtr.Zero", "AutoAway.get_IsWorkstationLocked");
					return true;
				}

				StringBuilder sb = new StringBuilder(260);
				GetUserObjectInformation(hDesktop, UOI_NAME,  sb, sb.Capacity, IntPtr.Zero);
				CloseDesktop(hDesktop);

				System.Diagnostics.Debug.WriteLine("sb = " + sb.ToString(), "AutoAway.get_IsWorkstationLocked");

				return sb.ToString() == "Winlogon";
			}
		}

		public static bool IsScreensaverRunning {
			get {
				bool retval = false;
				SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, 0, ref retval, false);

				System.Diagnostics.Debug.WriteLine("retval = " + retval.ToString(), "AutoAway.get_IsScreensaverRunning");

				return retval;
			}
		}

		private static System.Drawing.Point lastMousePosition;
		private static long mouseStationaryTimer = 0;

		public static long MouseStationaryTimer {
			get {
				if(Environment.OSVersion.Platform == PlatformID.Win32NT &&
					Environment.OSVersion.Version.Major >= 5) {

					LASTINPUTINFO lii = new LASTINPUTINFO();
					lii.cbSize = (uint)Marshal.SizeOf(lii);
					GetLastInputInfo(out lii);

					mouseStationaryTimer = (Environment.TickCount - lii.dwTime) / 1000;
					System.Diagnostics.Debug.WriteLine("mouseStationaryTimer = " + mouseStationaryTimer.ToString(), "AutoAway.get_MouseStationaryTimer [last input]");
					
				} else {
					if(System.Windows.Forms.Control.MousePosition != lastMousePosition) {
						lastMousePosition = System.Windows.Forms.Control.MousePosition;
						mouseStationaryTimer = 0;
					} else {
						mouseStationaryTimer += 5;
					}

					System.Diagnostics.Debug.WriteLine("mouseStationaryTimer = " + mouseStationaryTimer.ToString(), "AutoAway.get_MouseStationaryTimer [mouse]");
				}

				return mouseStationaryTimer;
			}
		}
	}
}
