using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;

namespace jachymko.Nyx.Wokno {

	[Serializable]
	public abstract class HistoryEvent {
		public DateTime time;
	}

	[Serializable]
	public class MailEvent : HistoryEvent {
		public Mail mail;

		public MailEvent(Mail m) {
			this.mail = m;
			this.time = m.Time;
		}
	}

	[Serializable]
	public class FriendEvent : HistoryEvent {
		public string userName;
		public bool online;

		public FriendEvent(string userName, bool online, DateTime time) {
			this.userName = userName;
			this.online = online;
			this.time = time;
		}
	}

	[Serializable]
	public class WriteupEvent : HistoryEvent {
		public string topic;
		public int id;

		public WriteupEvent(string topic, int id) {
			time = DateTime.Now;
			this.topic = topic;
			this.id = id;
		}
	}

	public class History {
		
		private const string UNENCRYPTED_FLAG = "FREE::";
		private const int MAX_HISTORY_LEN = 4096;

		private ArrayList events = new ArrayList();
		private string storePath;

		internal History(string storePath) {
			this.storePath = storePath;
			
			this.storePath = storePath;
			if(!File.Exists(storePath)) return;

			DataProtection dp = new DataProtection(DataProtection.Store.USE_USER_STORE);
			byte[] buff; 
	
			using(FileStream fs = new FileStream(storePath, FileMode.Open, FileAccess.Read)) {

				buff = new byte[fs.Length];
				fs.Read(buff, 0, unchecked((int)fs.Length));

				if(Encoding.ASCII.GetString(buff, 0, UNENCRYPTED_FLAG.Length) == UNENCRYPTED_FLAG) {
					Startup.Config.EncryptHistory = false;
					
					byte[] newbuff = new byte[buff.Length - UNENCRYPTED_FLAG.Length];
					Array.Copy(buff, UNENCRYPTED_FLAG.Length, newbuff, 0, newbuff.Length);
					buff = newbuff;
				} else {
					Startup.Config.EncryptHistory = true;
				}
			}

			if(Startup.Config.EncryptHistory) {
				buff = dp.Decrypt(buff);
			}

			MemoryStream ms = new MemoryStream(buff);
			BinaryFormatter fmt = new BinaryFormatter();

			events = (ArrayList)fmt.Deserialize(ms);
		}

		public void Save() {
			lock(events) {
				MemoryStream ms = new MemoryStream();
				DataProtection dp = new DataProtection(DataProtection.Store.USE_USER_STORE);
			
				BinaryFormatter fmt = new BinaryFormatter();
				fmt.Serialize(ms, events);
			
				byte[] buff = ms.ToArray();
			
				if(Startup.Config.EncryptHistory) {
					buff = dp.Encrypt(buff);
				}

				using(FileStream fs = new FileStream(storePath, FileMode.Create, FileAccess.Write, FileShare.None)) {
					byte[] flag = Encoding.ASCII.GetBytes(UNENCRYPTED_FLAG);

					if(!Startup.Config.EncryptHistory) {
						fs.Write(flag, 0, flag.Length);
					}
					fs.Write(buff, 0, buff.Length);
				}
			}
		}

		public FriendEvent AddFriendEvent(string userName, bool online) {
			FriendEvent fe = new FriendEvent(userName, online, DateTime.Now);
			events.Insert(0, fe);

			CheckLength();
			Save();

			return fe;
		}

		public WriteupEvent AddWriteupEvent(string topic, int id) {
			WriteupEvent we = new WriteupEvent(topic, id);
			events.Insert(0, we);

			CheckLength();
			Save();
			return we;
		}

		public MailEvent AddMailEvent(Mail mail) {
			MailEvent me = new MailEvent(mail);
			events.Insert(0, me);

			CheckLength();
			Save();

			return me;
		}

		public int Length {
			get {
				return events.Count;
			}
		}

		public HistoryEvent this[int i] {
			get {
				return (HistoryEvent)events[i];
			}
		}
		private void CheckLength() {
			if(events.Count > MAX_HISTORY_LEN) {
				events.RemoveRange(MAX_HISTORY_LEN, events.Count - MAX_HISTORY_LEN);
			}
		}
	}
}
