using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using System.Reflection;

namespace jachymko.Nyx.Wokno {

	public class IconCache {

		private DirectoryInfo iconCacheDir = Startup.EnsureDirectory("IconCache");
		private readonly Image defaultIcon = new Bitmap(Assembly.GetExecutingAssembly().GetManifestResourceStream("jachymko.Nyx.Wokno.Krizek.gif"));
		private const string NYX_ICON_STORE = "http://www.nyx.cz/ico/{0}/{1}.gif";

		public Image this[string userName] {
			get {
				Image icon = GetIconFromCache(userName);
				if(icon != null) {
					return icon;
				} else {
					HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(string.Format(NYX_ICON_STORE, userName[0], userName));
					if(Startup.Config.Proxy != null) {
						request.Proxy = Startup.Config.Proxy;
					}
					using(WebResponse response = request.GetResponse()) {
						using(Stream stream = response.GetResponseStream()) {
							CacheIcon(userName, stream);
							icon = GetIconFromCache(userName);
						}
					}

					//CacheIcon(userName, icon);						
					return icon;
				}
			}
		}

		private void CacheIcon(string userName, Stream iconStream) {
			string filePath = Path.Combine(iconCacheDir.FullName, userName + ".gif");

			using(BinaryWriter w = new BinaryWriter(new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.Read))) {
				
				int read = 0;
				do {
					byte[] buff = new byte[1024];
					read = iconStream.Read(buff, 0, buff.Length);

					if(read > 0) {
						w.Write(buff, 0, read);
					}
				} while(read > 0);
			}
		}

		private Image GetIconFromCache(string userName) {
			string filePath = Path.Combine(iconCacheDir.FullName, userName + ".gif");

			if(File.Exists(filePath)) {
				using(Stream stream = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.Read)) {
					return new Bitmap(stream);
				}
			}
			return null;
		}



		public IconCache() {
		}



	}
}
