using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Xml;

namespace jachymko.Nyx.Wokno {
	
	public class MailCache {

		private const string UNENCRYPTED_FLAG = "FREE::";
		private Hashtable users = Hashtable.Synchronized(new Hashtable());
		private string mailStorePath;

		public MailCache(string mailStorePath) {

			this.mailStorePath = mailStorePath;
			if(!File.Exists(mailStorePath)) return;

			DataProtection dp = new DataProtection(DataProtection.Store.USE_USER_STORE);
			byte[] buff; 
	
			using(FileStream fs = new FileStream(mailStorePath, FileMode.Open, FileAccess.Read)) {

				buff = new byte[fs.Length];
				fs.Read(buff, 0, unchecked((int)fs.Length));

				if(Encoding.ASCII.GetString(buff, 0, UNENCRYPTED_FLAG.Length) == UNENCRYPTED_FLAG) {
					Startup.Config.EncryptMailCache = false;
					
					byte[] newbuff = new byte[buff.Length - UNENCRYPTED_FLAG.Length];
					Array.Copy(buff, UNENCRYPTED_FLAG.Length, newbuff, 0, newbuff.Length);
					buff = newbuff;
				} else {
					Startup.Config.EncryptMailCache = true;
				}
			}

			if(Startup.Config.EncryptMailCache) {
				buff = dp.Decrypt(buff);
			}

			MemoryStream ms = new MemoryStream(buff);
			BinaryFormatter fmt = new BinaryFormatter();

			users = Hashtable.Synchronized((Hashtable)fmt.Deserialize(ms));
		}

		public void Save() {
			
			lock(users) {
				MemoryStream ms = new MemoryStream();
				DataProtection dp = new DataProtection(DataProtection.Store.USE_USER_STORE);
			
				BinaryFormatter fmt = new BinaryFormatter();
				fmt.Serialize(ms, users);
			
				byte[] buff = ms.ToArray();
			
				if(Startup.Config.EncryptMailCache) {
					buff = dp.Encrypt(buff);
				}

				using(FileStream fs = new FileStream(mailStorePath, FileMode.Create, FileAccess.Write, FileShare.None)) {
					byte[] flag = Encoding.ASCII.GetBytes(UNENCRYPTED_FLAG);

					if(!Startup.Config.EncryptMailCache) {
						fs.Write(flag, 0, flag.Length);
					}
					fs.Write(buff, 0, buff.Length);
				}
			}
		}

		public Mail[] this[string userName] {
			get {
				if(users[userName] != null) {
					ArrayList mails = (ArrayList)users[userName];
					return (Mail[])mails.ToArray(typeof(Mail));
				}
				return new Mail[0];
			}
		}
		public void AddMail(Mail mail) {
			if(users[mail.UserName] == null) {
				users[mail.UserName] = new ArrayList();
			}

			((ArrayList)users[mail.UserName]).Add(mail);

			Save();
		}

		public void AddSentMail(Mail mail, string recipient) {
			if(users[recipient] == null) {
				users[recipient] = new ArrayList();
			}
			
			((ArrayList)users[recipient]).Add(mail);
		}
	}
}
