using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace jachymko.Nyx.Wokno {

	public class MailForm : System.Windows.Forms.Form {

		private class User32 {
			[DllImport("user32", EntryPoint="FlashWindow")]
			public extern static bool FlashWindow(IntPtr hwnd, bool bInvert);
		}

		//HTML 4.01 Transitionalprivate const string msgFormat = "\r\n{2}\r\n\r\n";

		private string userName = string.Empty;
		private DateTime lastRefresh = DateTime.MinValue;
		private int oldMailCount = 0;
		private Image icon = null;
		private System.Windows.Forms.RichTextBox textMessages;
		private System.Windows.Forms.TextBox txtReply;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.LinkLabel lnkSendMail;
		private System.Windows.Forms.Label labelLastRefresh;
		private System.Windows.Forms.Label labelUserName;
		private System.ComponentModel.IContainer components;

//		[Obsolete]
//		internal int RefreshInterval {
//			set {
//				timerStatus.Interval = value;
//				timerStatus.Enabled = true;
//			}
//		}

		public static MailForm Get(string userName) {
			if(MainForm.mailForms[userName] == null || MainForm.mailForms[userName].IsDisposed) {
				MainForm.mailForms[userName] = new MailForm(userName);
			} 
			
			return MainForm.mailForms[userName];
		}

		private MailForm(string userName) {
			InitializeComponent();

			this.userName = userName;
			

//			timerStatus.Interval = Startup.Config.RefreshInterval;
			
			icon = Startup.IconCache[userName];
			labelUserName.Text = this.Text = userName;

			ReloadMails(false);
			timerStatus_Tick(this, EventArgs.Empty);
		}

		public string UserName {
			get {
				return this.userName;
			}
		}

		public void ReloadMails() {
			ReloadMails(true);
		}

		private void UpdateHeader() {
			if(MainForm.theOne.connStatus != MainForm.NyxStatus.Offline) {
				Friend[] friends = MainForm.witch.ActiveFriends;
				string fmt;

				for(int i = 0; i < friends.Length; i++) {
					if(friends[i].UserName == userName) {
						fmt = "[{0} :: online]";
						labelLastRefresh.Text = string.Format(fmt, Startup.FormatTime(friends[i].LastRefresh));
						return;
					}
				}

				labelLastRefresh.Text = "[offline/not a friend]";
			}
		}
		
		private void ReloadMails(bool doFlash) {


			const string dateTimeFormat = "{0} --- {1} {2} ---";
			Font boldFont = new Font(textMessages.Font, FontStyle.Bold);

			Mail[] mails = Startup.MailCache[userName];
			Array.Sort(mails);
			
			UpdateHeader();

			if(mails.Length == oldMailCount) return;
			
			textMessages.Text = string.Empty;
			for(int i = mails.Length - 1; i >= 0; i--) {
				string dateTime = string.Format(dateTimeFormat, mails[i].UserName, mails[i].Time.ToShortTimeString(), mails[i].Time.ToShortDateString());
				string text = FormatHtml(mails[i].Text);

				textMessages.SelectionFont = boldFont;
				textMessages.AppendText(dateTime);
				
				textMessages.SelectionFont = textMessages.Font;
				textMessages.AppendText(Environment.NewLine);
				textMessages.AppendText(text);
				textMessages.AppendText(Environment.NewLine);
				textMessages.AppendText(Environment.NewLine);
			}

			textMessages.SelectionStart = 0;
			textMessages.SelectionLength = 0;

			if(mails.Length > oldMailCount && doFlash) {
				FlashWindow();
			}
			oldMailCount = mails.Length;
			
		}

		public static string FormatHtml(string html) {
			html = html.Replace("<br>", Environment.NewLine);
			html = html.Replace("<br />", Environment.NewLine);
			html = html.Replace("<br>", Environment.NewLine);
			html = html.Replace("<br />", Environment.NewLine);

			bool inTag = false; 
			StringBuilder txt = new StringBuilder(html.Length);

			for(int i = 0; i < html.Length; i++) {
				if(html[i] == '<') {
					inTag = true;
					continue;
				}
				if(html[i] == '>') {
					inTag = false;
					continue;
				}

				if(inTag) {
					continue;
				}

				txt.Append(html[i]);
			}

			return System.Web.HttpUtility.HtmlDecode(txt.ToString());
		}

		public void FlashWindow() {
			User32.FlashWindow(Handle, true);
		}


		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MailForm));
			this.textMessages = new System.Windows.Forms.RichTextBox();
			this.txtReply = new System.Windows.Forms.TextBox();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.panel1 = new System.Windows.Forms.Panel();
			this.lnkSendMail = new System.Windows.Forms.LinkLabel();
			this.labelLastRefresh = new System.Windows.Forms.Label();
			this.labelUserName = new System.Windows.Forms.Label();
			this.panel1.SuspendLayout();
			this.SuspendLayout();
			// 
			// textMessages
			// 
			this.textMessages.Dock = System.Windows.Forms.DockStyle.Fill;
			this.textMessages.Location = new System.Drawing.Point(0, 64);
			this.textMessages.Name = "textMessages";
			this.textMessages.ReadOnly = true;
			this.textMessages.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.ForcedVertical;
			this.textMessages.Size = new System.Drawing.Size(520, 109);
			this.textMessages.TabIndex = 1;
			this.textMessages.Text = "";
			// 
			// txtReply
			// 
			this.txtReply.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.txtReply.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.txtReply.Location = new System.Drawing.Point(0, 181);
			this.txtReply.Multiline = true;
			this.txtReply.Name = "txtReply";
			this.txtReply.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.txtReply.Size = new System.Drawing.Size(520, 120);
			this.txtReply.TabIndex = 0;
			this.txtReply.Text = "";
			this.txtReply.KeyUp += new System.Windows.Forms.KeyEventHandler(this.txtReply_KeyUp);
			// 
			// splitter1
			// 
			this.splitter1.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.splitter1.Location = new System.Drawing.Point(0, 173);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(520, 8);
			this.splitter1.TabIndex = 3;
			this.splitter1.TabStop = false;
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.lnkSendMail);
			this.panel1.Controls.Add(this.labelLastRefresh);
			this.panel1.Controls.Add(this.labelUserName);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Top;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(520, 64);
			this.panel1.TabIndex = 2;
			this.panel1.Paint += new System.Windows.Forms.PaintEventHandler(this.panel1_Paint);
			// 
			// lnkSendMail
			// 
			this.lnkSendMail.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.lnkSendMail.AutoSize = true;
			this.lnkSendMail.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lnkSendMail.Location = new System.Drawing.Point(424, 40);
			this.lnkSendMail.Name = "lnkSendMail";
			this.lnkSendMail.Size = new System.Drawing.Size(88, 17);
			this.lnkSendMail.TabIndex = 0;
			this.lnkSendMail.TabStop = true;
			this.lnkSendMail.Text = "Reply in browser";
			this.lnkSendMail.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.lnkSendMail_LinkClicked);
			// 
			// labelLastRefresh
			// 
			this.labelLastRefresh.Location = new System.Drawing.Point(56, 34);
			this.labelLastRefresh.Name = "labelLastRefresh";
			this.labelLastRefresh.Size = new System.Drawing.Size(200, 23);
			this.labelLastRefresh.TabIndex = 2;
			this.labelLastRefresh.Text = "[03:10 :: online]";
			// 
			// labelUserName
			// 
			this.labelUserName.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.labelUserName.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.labelUserName.Location = new System.Drawing.Point(56, 8);
			this.labelUserName.Name = "labelUserName";
			this.labelUserName.Size = new System.Drawing.Size(456, 23);
			this.labelUserName.TabIndex = 1;
			this.labelUserName.Text = "JACHYMKO_TEST";
			// 
			// MailForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(520, 301);
			this.Controls.Add(this.textMessages);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.txtReply);
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "MailForm";
			this.Text = "JACHYMKO_TEST";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MailForm_Closing);
			this.panel1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void timerStatus_Tick(object sender, System.EventArgs e) {

		}

		private void MailForm_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			MainForm.mailForms[userName] = null;

			// moved to MainForm.witch_NewMail
			//Startup.MailCache.Save();
		}

		[Obsolete]
		public void AttachNotifyEvent(NotifyEvent ev) {
			ev.Activated += new EventHandler(NotifyEventClick);
		}
		private void NotifyEventClick(object sender, EventArgs args) {
			Show();
			Focus();
			Activate();
		}

		public void AttachBallon(NotifyIconEx nie) {
			nie.BalloonClick += new EventHandler(nie_BalloonClick);
			nie.BalloonClosed += new EventHandler(nie_BalloonClosed);
		}

		private void nie_BalloonClick(object sender, EventArgs e) {
			((NotifyIconEx)sender).BalloonClick -= new EventHandler(nie_BalloonClick);
			((NotifyIconEx)sender).BalloonClosed -= new EventHandler(nie_BalloonClosed);

			Show();
			Focus();
		}

		private void nie_BalloonClosed(object sender, EventArgs e) {
			((NotifyIconEx)sender).BalloonClick -= new EventHandler(nie_BalloonClick);
			((NotifyIconEx)sender).BalloonClosed -= new EventHandler(nie_BalloonClosed);
		}

		private void lnkSendMail_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e) {
			WebActions.SendMail(userName);
		}

		private void txtReply_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e) {
			if((e.KeyCode == Keys.Enter && e.Control) || (e.KeyCode == Keys.S && e.Alt)) {

				txtReply.ReadOnly = true;
				txtReply.Update();

				string mailText = txtReply.Text.Trim();

				txtReply.Text = string.Empty;
				txtReply.ReadOnly = false;
				txtReply.Update();

				if(mailText.Length == 0) return;

				try {
					Mail m  = MainForm.witch.SendMail(userName, mailText);
					Startup.MailCache.AddSentMail(m, userName);
				} catch(Exception exc) {
					MessageBox.Show(this, exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				
				ReloadMails(false);
			}
		}

		private void panel1_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			if(icon == null) return;

			e.Graphics.DrawImage(icon, 8, 8, 40, 50);
		}
	}
}
