//#define TEST

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;

namespace jachymko.Nyx.Wokno {
	
	public class Startup {
	
		// [exe dir]/NyxWokno
		private static readonly Config config;
		private static readonly DirectoryInfo myDir = new DirectoryInfo(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "NyxWokno"));
		private static readonly IconCache iconCache = new IconCache();
		private static readonly MailCache mailCache;
		private static readonly History history;
		private static readonly BookWatch bookWatch;
		
		static Startup() {
			EnsureDirectory();

			bookWatch = new BookWatch();
			config = new Config(Path.Combine(myDir.FullName, "NyxWokno.config"));
			mailCache = new MailCache(Path.Combine(myDir.FullName, "MailCache.bin"));
			history = new History(Path.Combine(myDir.FullName, "History.bin"));
		}

		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();

#if TEST
			Application.Run(new TestForm());
#else
			MainForm mf = new MainForm();
			MainForm.theOne = mf;

			Application.Run(mf);
#endif
		}

		internal static DirectoryInfo EnsureDirectory() {
			// ensure directory [exe dir]/NyxWokno
			if(!myDir.Exists) myDir.Create();

			return myDir;
		}

		internal static DirectoryInfo EnsureDirectory(string directoryName) {
			// ensure directory [exe dir]/NyxWokno
			if(!myDir.Exists) myDir.Create();
			
			// ensure directory ./NyxWokno/[directoryName]
			DirectoryInfo di = new DirectoryInfo(Path.Combine(myDir.FullName, directoryName));
			if(!di.Exists) di.Create();

			return di;
		}
		internal static string FormatTime(DateTime time) {
			if(Startup.Config.ShowAltTime) {
				TimeSpan ts = (DateTime.Now - time);

				int min = (int)Math.Floor(ts.TotalMinutes);
				int sec = (int)ts.Seconds;

				string m, s;
				if(min < 0)
					m = "00";
				else if(min < 10)
					m = "0" + min.ToString();
				else
					m = min.ToString();

				if(sec < 0)
					s = "00";
				else if(sec < 10) 
					s = "0" + sec.ToString();
				else
					s = sec.ToString();

				return string.Format("{0}:{1}", m, s);
			} else {
				return time.ToLongTimeString();
			}
		}
		internal static IconCache IconCache {
			get{
				return iconCache;
			}
		}
		internal static MailCache MailCache {
			get {
				return mailCache;
			}
		}
		internal static Config Config {
			get {
				return config;
			}
		}
		internal static History History {
			get {
				return history;
			}
		}
		internal static BookWatch WatchTopic {
			get {
				return bookWatch;
			}
		}
	}
}
