using System;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Threading;
using System.Web.UI;

namespace jachymko.Nyx.Wokno {

	public class WebActions {

		private const string NYX_URL       = "http://www.nyx.cz/code/index.php";
		private const string TOPIC_URL     = NYX_URL + "?l=topic;id={1};n={0}";
		private const string SEND_MAIL_URL = NYX_URL + "?l=mail;recipient={1};n={0}";
		private const string COMMENT_URL   = NYX_URL + "?l=people;l2=3;friendname={1};n={0}";
		private const string INFO_URL      = NYX_URL + "?l=pref;user={1};n={0}";
		private const string SEARCH_URL    = NYX_URL + "?l=topics;l2=3;wu_author={1};n={0}";

		private const string BOOKMARKS_URL = NYX_URL + "?l=pref;l2=8;id_user={1};n={0}";
		private const string FRIENDS_URL   = NYX_URL + "?l=pref;l2=5;id_user={1};n={0}";
		private const string COMMENTS_URL  = NYX_URL + "?l=pref;l2=4;id_user={1};n={0}";

		private const string GATEWAY_URL   = NYX_URL + "?l=gate;n={0}";
		private const string PREFS_URL	   = NYX_URL + "?l=pref;l2=2;n={0}";

		private static Random rnd = new Random(Environment.TickCount);

		private WebActions() {
		}

		public static void SendMail(string userName) {
			string url = string.Format(SEND_MAIL_URL, GetRandom(), userName.ToUpper());
			OpenInBrowser(url);
		}
		public static void ViewTopic(int topicId) {
			string url = string.Format(TOPIC_URL, GetRandom(), topicId.ToString());
			OpenInBrowser(url);
		}

		public static void ChangeRemoveComment(string userName) {
			string url = string.Format(COMMENT_URL, GetRandom(), userName.ToUpper());
			OpenInBrowser(url);
		}

		public static void ViewInfo(string userName) {
			string url = string.Format(INFO_URL, GetRandom(), userName.ToUpper());
			OpenInBrowser(url);
		}

		public static void Search(string userName) {
			string url = string.Format(SEARCH_URL, GetRandom(), userName.ToUpper());
			OpenInBrowser(url);
		}

		public static void Preferences() {
			string url = string.Format(PREFS_URL, GetRandom());
			OpenInBrowser(url);
		}

		public static void Gateway() {
			string url = string.Format(GATEWAY_URL, GetRandom());
			OpenInBrowser(url);
		}

		public static void Bookmarks(int userId) {
			string url = string.Format(BOOKMARKS_URL, GetRandom(), userId.ToString());
			OpenInBrowser(url);
		}

		public static void Friends(int userId) {
			string url = string.Format(FRIENDS_URL, GetRandom(), userId.ToString());
			OpenInBrowser(url);
		}

		public static void Comments(int userId) {
			string url = string.Format(COMMENTS_URL, GetRandom(), userId.ToString());
			OpenInBrowser(url);
		}

		private static void OpenInBrowser(string url) {

			if(Startup.Config.WebPassword.Length == 0) {
				Process.Start(url);
				return;
			}

			string tmp = Path.GetTempFileName() + ".htm";

			if(url[url.Length - 1] != ';') {
				url += ';';
			}

			HtmlTextWriter output = new HtmlTextWriter(
										new StreamWriter(
											File.Open(tmp, FileMode.Create, FileAccess.Write, FileShare.Read), Encoding.UTF8
										)
									);
	
			output.WriteBeginTag("html");
			output.Write(HtmlTextWriter.TagRightChar);
			
			output.WriteBeginTag("body");
			output.WriteAttribute("onload", "document.getElementById('login').submit();", false);
			output.Write(HtmlTextWriter.TagRightChar);
			
			output.WriteBeginTag("form");
			output.WriteAttribute("id", "login", false);
			output.WriteAttribute("name", "login", false);
			output.WriteAttribute("action", url + "login=1", false);
			output.WriteAttribute("method", "post", false);
			output.Write(HtmlTextWriter.TagRightChar);

			output.WriteBeginTag("input");
			output.WriteAttribute("type", "hidden", false);
			output.WriteAttribute("name", "loguser", false);
			output.WriteAttribute("value", Startup.Config.UserName, false);
			output.Write(HtmlTextWriter.TagRightChar);

			output.WriteBeginTag("input");
			output.WriteAttribute("type", "hidden", false);
			output.WriteAttribute("name", "logpass", false);
			output.WriteAttribute("value", Startup.Config.WebPassword, false);
			output.Write(HtmlTextWriter.TagRightChar);

			output.WriteBeginTag("input");
			output.WriteAttribute("type", "hidden", false);
			output.WriteAttribute("name", "loginv", false);
			output.WriteAttribute("value", "0", false);
			output.Write(HtmlTextWriter.TagRightChar);

			output.WriteEndTag("form");
			output.WriteEndTag("body");
			output.WriteEndTag("html");
			output.Flush();

			Process.Start(tmp);
			ThreadPool.QueueUserWorkItem(new WaitCallback(DeleteTmpFile), new TmpFileInfo(output, tmp));
		}

		private struct TmpFileInfo {
			public HtmlTextWriter output;
			public string fileName;

			public TmpFileInfo(HtmlTextWriter output, string fileName) {
				this.output = output;
				this.fileName = fileName;
			}
		}

		private static void DeleteTmpFile(object tmpFile) {
			Thread.Sleep(10000);
			TmpFileInfo info = (TmpFileInfo)tmpFile;

			info.output.Close();
			File.Delete(info.fileName);
		}

		private static string GetRandom() {
			return Convert.ToString(rnd.Next(), 16).PadLeft(4, '0');
		}
	}
}
